@@ -41,6 +41,7 @@ class PikaEventBroker(EventBroker):
         event_loop: Optional[AbstractEventLoop] = None,
         connection_attempts: int = 20,
         retry_delay_in_seconds: float = 5,
+        exchange_name: Text = RABBITMQ_EXCHANGE,
         **kwargs: Any,
     ):
         """Initialise RabbitMQ event broker.
@@ -62,6 +63,8 @@ class PikaEventBroker(EventBroker):
             connection_attempts: Number of attempts for connecting to RabbitMQ before
                 an exception is thrown.
             retry_delay_in_seconds: Time in seconds between connection attempts.
+            exchange_name: Exchange name to which the queues binds to.
+                If nothing is mentioned then the default exchange name would be used.
         """
         super().__init__()
         _set_pika_log_levels(log_level)
@@ -74,6 +77,7 @@ class PikaEventBroker(EventBroker):
         self.raise_on_failure = raise_on_failure
         self._connection_attempts = connection_attempts
         self._retry_delay_in_seconds = retry_delay_in_seconds
+        self.exchange_name = exchange_name
 
         # Unpublished messages which hopefully will be published later 🤞
         self._unpublished_events: Deque[Dict[Text, Any]] = deque()
@@ -146,7 +150,9 @@ class PikaEventBroker(EventBroker):
         logger.info(f"RabbitMQ connection to '{self.host}' was established.")
 
         channel = await self._connection.channel()
-        logger.debug("RabbitMQ channel was opened. Declaring fanout exchange.")
+        logger.debug(
+            f"RabbitMQ channel was opened. Declaring fanout exchange '{self.exchange_name}'."
+        )
 
         self._exchange = await self._set_up_exchange(channel)
 
@@ -201,7 +207,7 @@ class PikaEventBroker(EventBroker):
         self, channel: aio_pika.RobustChannel
     ) -> aio_pika.Exchange:
         exchange = await channel.declare_exchange(
-            RABBITMQ_EXCHANGE, type=aio_pika.ExchangeType.FANOUT
+            self.exchange_name, type=aio_pika.ExchangeType.FANOUT
         )
 
         await asyncio.gather(
