@@ -12,6 +12,7 @@ from rasa.nlu.config import RasaNLUModelConfig
 from rasa.shared.exceptions import InvalidConfigException
 from rasa.shared.nlu.training_data.training_data import TrainingData
 from rasa.shared.nlu.training_data.message import Message
+from rasa.nlu.constants import COMPONENT_INDEX
 import rasa.shared.utils.io
 
 if typing.TYPE_CHECKING:
@@ -388,26 +389,38 @@ class Component(metaclass=ComponentMetaclass):
     the pipeline to do intent classification.
     """
 
-    # Component class name is used when integrating it in a
-    # pipeline. E.g. ``[ComponentA, ComponentB]``
-    # will be a proper pipeline definition where ``ComponentA``
-    # is the name of the first component of the pipeline.
     @property
     def name(self) -> Text:
-        """Access the class's property name from an instance."""
+        """Returns the name of the component to be used in the model configuration.
 
+        Component class name is used when integrating it in a
+        pipeline. E.g. `[ComponentA, ComponentB]`
+        will be a proper pipeline definition where `ComponentA`
+        is the name of the first component of the pipeline.
+        """
         return type(self).name
 
-    # Which components are required by this component.
-    # Listed components should appear before the component itself in the pipeline.
+    @property
+    def unique_name(self) -> Text:
+        """Gets a unique name for the component in the pipeline.
+
+        The unique name can be used to distinguish components in
+        a pipeline, e.g. when the pipeline contains multiple
+        featurizers of the same type.
+        """
+        index = self.component_config.get(COMPONENT_INDEX)
+        return self.name if index is None else f"component_{index}_{self.name}"
+
     @classmethod
     def required_components(cls) -> List[Type["Component"]]:
-        """Specify which components need to be present in the pipeline.
+        """Specifies which components need to be present in the pipeline.
+
+        Which components are required by this component.
+        Listed components should appear before the component itself in the pipeline.
 
         Returns:
-            The list of class names of required components.
+            The class names of the required components.
         """
-
         return []
 
     # Defines the default configuration parameters of a component
@@ -452,7 +465,7 @@ class Component(metaclass=ComponentMetaclass):
 
     @classmethod
     def required_packages(cls) -> List[Text]:
-        """Specify which python packages need to be installed.
+        """Specifies which python packages need to be installed.
 
         E.g. ``["spacy"]``. More specifically, these should be
         importable python package names e.g. `sklearn` and not package
@@ -464,7 +477,6 @@ class Component(metaclass=ComponentMetaclass):
         Returns:
             The list of required package names.
         """
-
         return []
 
     @classmethod
@@ -476,7 +488,7 @@ class Component(metaclass=ComponentMetaclass):
         cached_component: Optional["Component"] = None,
         **kwargs: Any,
     ) -> "Component":
-        """Load this component from file.
+        """Loads this component from file.
 
         After a component has been trained, it will be persisted by
         calling `persist`. When the pipeline gets loaded again,
@@ -494,7 +506,6 @@ class Component(metaclass=ComponentMetaclass):
         Returns:
             the loaded component
         """
-
         if cached_component:
             return cached_component
 
@@ -515,7 +526,6 @@ class Component(metaclass=ComponentMetaclass):
         Returns:
             The created component.
         """
-
         # Check language supporting
         language = config.language
         if not cls.can_handle_language(language):
@@ -525,7 +535,7 @@ class Component(metaclass=ComponentMetaclass):
         return cls(component_config)
 
     def provide_context(self) -> Optional[Dict[Text, Any]]:
-        """Initialize this component for a new pipeline.
+        """Initializes this component for a new pipeline.
 
         This function will be called before the training
         is started and before the first message is processed using
@@ -540,7 +550,6 @@ class Component(metaclass=ComponentMetaclass):
         Returns:
             The updated component configuration.
         """
-
         pass
 
     def train(
@@ -549,7 +558,7 @@ class Component(metaclass=ComponentMetaclass):
         config: Optional[RasaNLUModelConfig] = None,
         **kwargs: Any,
     ) -> None:
-        """Train this component.
+        """Trains this component.
 
         This is the components chance to train itself provided
         with the training data. The component can rely on
@@ -561,16 +570,13 @@ class Component(metaclass=ComponentMetaclass):
         of components previous to this one.
 
         Args:
-            training_data:
-                The :class:`rasa.shared.nlu.training_data.training_data.TrainingData`.
+            training_data: The :class:`rasa.shared.nlu.training_data.training_data.TrainingData`.
             config: The model configuration parameters.
-
         """
-
         pass
 
     def process(self, message: Message, **kwargs: Any) -> None:
-        """Process an incoming message.
+        """Processes an incoming message.
 
         This is the components chance to process an incoming
         message. The component can rely on
@@ -583,13 +589,11 @@ class Component(metaclass=ComponentMetaclass):
 
         Args:
             message: The :class:`rasa.shared.nlu.training_data.message.Message` to process.
-
         """
-
         pass
 
     def persist(self, file_name: Text, model_dir: Text) -> Optional[Dict[Text, Any]]:
-        """Persist this component to disk for future loading.
+        """Persists this component to disk for future loading.
 
         Args:
             file_name: The file name of the model.
@@ -598,7 +602,6 @@ class Component(metaclass=ComponentMetaclass):
         Returns:
             An optional dictionary with any information about the stored model.
         """
-
         pass
 
     @classmethod
@@ -619,10 +622,10 @@ class Component(metaclass=ComponentMetaclass):
         Returns:
             A unique caching key.
         """
-
         return None
 
     def __getstate__(self) -> Any:
+        """Gets a copy of picklable parts of the component."""
         d = self.__dict__.copy()
         # these properties should not be pickled
         if "partial_processing_context" in d:
